--[[
**********************************************************************
MagicDKP Bidder - Addon to handle the DKP Bidding process
**********************************************************************
This file is part of MagicDKP, a World of Warcraft Addon

MagicDKP is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MagicDKP is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with MagicDKP.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************************
]]


-- Custom AceGUI-3.0 widgets
local GUI = LibStub("AceGUI-3.0")

do
   -- the header in the DKP info
   local Type = "MDKPHeader"
   local counter = 1
   local function OnAcquire(self)
      self:SetData()
   end

   local function OnRelease(self)
      self.frame:ClearAllPoints()
      self.frame:Hide()
   end

   local function SetData(self, player, bid, prio, current)
      self.player:SetText(player or "")
      self.bid:SetText(bid and tostring(bid) or "")
      self.priority:SetText(prio or "")
      self.current:SetText(current or "")
   end

   local function ConstructCell(self, name, width, left, right)
      local widgetName = Type..counter..name.."Button"

      local button = CreateFrame("Button", widgetName, self.frame, "MDKPBidderHeaderTemplate")
      if not left then
	 button:SetPoint("LEFT", self.frame ,"LEFT",-1,0)
      else
	 button:SetPoint("LEFT", left ,"RIGHT",-1,0)
      end
      if right then button:SetPoint("RIGHT", right, "RIGHT", 1, 0) end
      if width then button:SetWidth(width) end
      button:SetHeight(24)
      self[name] = button
      button.name = name
      button.obj = self
      button.mid = _G[widgetName.."Middle"]
      if width then button.mid:SetWidth(width-10) end

      button:SetScript("OnSizeChanged", function(self, width, height)
					   self.mid:SetWidth(width-10)
					end)
      button:SetScript("OnClick", function(self)
				     self.obj:Fire("OnRowClick", self.name)
				  end)
   end
   
   local function Constructor(self)
      local frame = CreateFrame("Frame", nil, UIParent)
      local self = {}
      self.type = Type
      
      self.OnRelease = OnRelease
      self.OnAcquire = OnAcquire
      self.SetData = SetData
      self.SetPriority = SetPriority

      self.frame = frame
      frame.obj = self
		
      frame:SetHeight(18)

      ConstructCell(self, "player", 120)
      ConstructCell(self, "bid", 60, self.player)
      ConstructCell(self, "current", 60, self.bid)
      ConstructCell(self, "priority", nil, self.current, frame)

      self.width = "fill"

      counter = counter + 1

      GUI:RegisterAsWidget(self)
      return self
   end
   
   GUI:RegisterWidgetType(Type,Constructor,1)
end


do
   -- A row of dkp bidder information
   local Type = "MDKPRow"
   local function OnAcquire(self)
      self:SetData()
      self.selected:Hide()
      self.highlight:Hide()
   end

   local function OnRelease(self)
      self.highlight:Hide()
      self.selected:Hide()
      self.frame:ClearAllPoints()
      self.frame:Hide()
   end

   local function SetPlayer(self, player, color)
      self.player:SetText(player or "")
      if color then
	 self.player:SetTextColor(color.r, color.g, color.b)
      else
	 self.player:SetTextColor(0.7, 0.7, 0.7)
      end
   end

   local function SetBid(self, bid)
      local cur = tonumber(self.current:GetText())
      local bidi = tonumber(bid)
      if bidi and cur and bidi > cur then
	 self.bid:SetTextColor(1.0, 0.4, 0)
      else
	 self.bid:SetTextColor(0.8, 1.0, 0.4)
      end
      self.bid:SetText(bid and tostring(bid) or "")
   end

   local function SetPriority(self, prio)
      self.priority:SetText(prio or "")
   end

   local function SetData(self, player, bid, prio, color)
      SetPlayer(self, player, color)
      SetBid(self, bid)
      SetPriority(self, prio)
   end

   local function SetCurrent(self, current)
      self.current:SetText(current or "")
   end
   
   local function ConstructCell(self, name, width, left, right)
      local backdropFrame = CreateFrame("Frame", nil, self.frame)
      if not left then
	 backdropFrame:SetPoint("LEFT", self.frame ,"LEFT",1,0)
      else
	 backdropFrame:SetPoint("LEFT", left ,"RIGHT",1,0)
      end
      if right then backdropFrame:SetPoint("RIGHT", right, "RIGHT", -1, 0) end
      if width then backdropFrame:SetWidth(width) end
      backdropFrame:SetHeight(18)
      self[name.."Frame"] = backdropFrame
      backdropFrame.obj = self
      
      local label = backdropFrame:CreateFontString(nil,"OVERLAY", "GameFontNormal")
      label:SetPoint("TOPLEFT", backdropFrame, "TOPLEFT", 3, 0)
      label:SetPoint("BOTTOMRIGHT", backdropFrame, "BOTTOMRIGHT", -10, 0)
      label:SetJustifyH("LEFT")
      self[name] = label
      label.obj = self
   end

   local function SetSelected(self, selected)
      if selected then
	 self.selected:Show()
      else
	 self.selected:Hide()
      end
   end
   
   local function Constructor(self)
      local frame = CreateFrame("Button",nil,UIParent)
      local self = {}
      self.type = Type
      
      self.OnRelease = OnRelease
      self.OnAcquire = OnAcquire
      self.SetPlayer = SetPlayer
      self.SetCurrent = SetCurrent
      self.SetBid = SetBid
      self.SetData = SetData
      self.SetPriority = SetPriority
      self.SetSelected = SetSelected

      self.frame = frame
      frame.obj = self
		
      frame:SetHeight(18)
      frame:SetScript("OnEnter", function(self) self.obj.highlight:Show() end)
      frame:SetScript("OnLeave", function(self) self.obj.highlight:Hide() end)	
      frame:SetScript("OnClick", function(self) self.obj:Fire("OnRowClick", self.obj.player:GetText()) end)
								 
      local highlight = frame:CreateTexture(nil, "HIGHLIGHT")
      highlight:SetTexture("Interface/Buttons/UI-Listbox-Highlight2")
      highlight:SetBlendMode("ADD")
      highlight:SetVertexColor(0.7,0.5,0,0.4)
      highlight:SetAllPoints()
      self.highlight = highlight

      local selected = frame:CreateTexture(nil, "BACKGROUND")
      selected:SetTexture("Interface/Buttons/UI-Listbox-Highlight2")
      selected:SetVertexColor(0,0.7,1,0.7)
      selected:SetAllPoints()
      self.selected = selected

      frame:EnableMouse(true)

      ConstructCell(self, "player", 120)
      ConstructCell(self, "bid", 60, self.playerFrame)
      ConstructCell(self, "current", 60, self.bidFrame)
      ConstructCell(self, "priority", nil, self.currentFrame, frame)


      local cancel = CreateFrame("Button", nil, frame)
      cancel:SetNormalTexture("Interface\\Buttons\\CancelButton-Up")
      cancel:SetPushedTexture("Interface\\Buttons\\CancelButton-Down")
      cancel:SetHighlightTexture("Interface\\Buttons\\CancelButton-Highlight", "AD")
      cancel:SetScript("OnClick", function(self) self.obj:Fire("OnRowDelete", self.obj.player:GetText()) end)
      cancel:SetWidth(24)
      cancel:SetHeight(24)
      cancel:SetPoint("RIGHT", frame, "RIGHT")
      cancel.obj = self
      self.cancel = cancel
      
      self.bid:SetJustifyH("RIGHT")

      self.width = "fill"

      GUI:RegisterAsWidget(self)
      return self
   end
   
   GUI:RegisterWidgetType(Type,Constructor,1)
end
